/*******************************************************************/
/*                                                                 */
/*  FILE     generate.C                                            */
/*  MODULE   generate                                              */
/*  PROGRAM  SFST                                                  */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*                                                                 */
/*  PURPOSE  prints automata                                       */
/*                                                                 */
/*******************************************************************/

#include "fst.h"

using std::vector;

namespace SFST {

  /********************  Gen  ***************************************/
  
  class Gen {
  public:
    Node *node;
    Index previous;
    Label label;

    Gen( Node *n, Label l=Label::epsilon , Index p=undef ) 
      : node(n), previous(p), label(l) {}

    void print( vector<Gen> &paths, FILE *file, Alphabet &alphabet ) {
      if (previous != undef) {
	paths[previous].print( paths, file, alphabet );
	fputs(alphabet.write_label(label), file);
      }
    }
  };


  /*******************************************************************/
  /*                                                                 */
  /*  Transducer::generate                                           */
  /*                                                                 */
  /*******************************************************************/

  void Transducer::generate( FILE *file, int max )

  {
    vector<Gen> paths;
    paths.push_back(Gen(root_node()));
 
    int n=0;
    for( size_t i=0; i<paths.size(); i++ ) {
      // fprintf(stderr,">>> %lu\n", i);
      Gen &gen = paths[i];
      Node *node = gen.node;
      if (node->is_final()) {
	gen.print( paths, file, alphabet );
	fputc('\n',file);
	if (++n == max)
	  return;
      }

      for( ArcsIter p(node->arcs()); p; p++ ) {
	Arc *arc=p;
	paths.push_back( Gen( arc->target_node(), arc->label(), (Index)i ) );
      }
    }
  }
}
